local _, DungeonOrganizer = ...;

local min_level = 60;

function DungeonOrganizer:InitDB()
	local t = {};
	t.alts = {};
	return t;
end

function DungeonOrganizer:OnLoad()
	self.main_frame:UnregisterEvent("ADDON_LOADED");
	
	DungeonOrganizerDB = DungeonOrganizerDB or self:InitDB();
end

function DungeonOrganizer:ReportData(chatType, message)
	if message == '!key' then 
		local isMax, _, _, dungeonId, _, level = self:CollectData();
		if not isMax then return end;
		if not dungeonId then return end;
	
		local dungeonName = self:GetKeystoneName(dungeonId);
		SendChatMessage(dungeonName..' '..level, chatType, nil, nil);
	end
	
	if message == '!keys' then 
		self:CollectData();
		self:ResetAlts();
		local db = DungeonOrganizerDB
		if not db then return end;

		local messageOut = '';
		local needPipe = false;
		for guid, alt in pairs(db.alts) do
			if alt.dungeon ~= 'Unknown' then
				if needPipe then
					messageOut = messageOut..', '
				end
				needPipe = true
				messageOut = messageOut..(alt.name)..'-'..(alt.dungeon)..' '..(alt.level)
			end
		end

		if messageOut ~= '' then
			SendChatMessage(messageOut, chatType, nil, nil);
		end
	end
end

function DungeonOrganizer:GetKeystoneName(dungeonId)
	print(dungeonId);
	return C_ChallengeMode.GetMapUIInfo(dungeonId) or "Unknown Dungeon";
end

function DungeonOrganizer:ResetAlts()
	local db = DungeonOrganizerDB
	if not db then return end;

	for guid, v in pairs(db.alts) do
		local expiry = db.alts[guid].expires or 0;
		local char_table = db.alts[guid];
		if time() > expiry then
			char_table.dungeon = "Unknown";
			char_table.level = "?";
			char_table.expires = self:GetNextWeeklyResetTime();
		end
	end
end

function DungeonOrganizer:CollectData()
	DungeonOrganizer:ResetAlts()
	if UnitLevel('player') < min_level then return false end;
	
	local guid = UnitGUID('player');
	local name = UnitName('player')
	local dungeonId = C_MythicPlus.GetOwnedKeystoneChallengeMapID();
	local expires = self:GetNextWeeklyResetTime();
	local level = C_MythicPlus.GetOwnedKeystoneLevel();

	if dungeonId then
		DungeonOrganizerDB.alts[guid] = { name = name; dungeon = self:GetKeystoneName(dungeonId); expires = expires; level = level; }
	end

	if not dungeonId then
		DungeonOrganizerDB.alts[guid] = { name = name; dungeon = "Unknown"; expires = expires; level = "?"; }
	end
	
	return true, guid, name, dungeonId, expires, level
end

function DungeonOrganizer:Init(frame)
	self.main_frame = frame
end

function DungeonOrganizer:GetRegion()
	if not self.region then
		local reg
		reg = GetCVar("portal")
		if reg == "public-test" then -- PTR uses US region resets, despite the misleading realm name suffix
			reg = "US"
		end
		if not reg or #reg ~= 2 then
			local gcr = GetCurrentRegion()
			reg = gcr and ({ "US", "KR", "EU", "TW", "CN" })[gcr]
		end
		if not reg or #reg ~= 2 then
			reg = (GetCVar("realmList") or ""):match("^(%a+)%.")
		end
		if not reg or #reg ~= 2 then -- other test realms?
			reg = (GetRealmName() or ""):match("%((%a%a)%)")
		end
		reg = reg and reg:upper()
		if reg and #reg == 2 then
			self.region = reg
		end
	end
	return self.region
end

function DungeonOrganizer:GetServerOffset()
	local serverDay = C_DateAndTime.GetCurrentCalendarTime().weekday - 1 -- 1-based starts on Sun
	local localDay = tonumber(date("%w")) -- 0-based starts on Sun
	local serverHour, serverMinute = GetGameTime()
	local localHour, localMinute = tonumber(date("%H")), tonumber(date("%M"))
	if serverDay == (localDay + 1)%7 then -- server is a day ahead
		serverHour = serverHour + 24
	elseif localDay == (serverDay + 1)%7 then -- local is a day ahead
		localHour = localHour + 24
	end
	local server = serverHour + serverMinute / 60
	local localT = localHour + localMinute / 60
	local offset = floor((server - localT) * 2 + 0.5) / 2
	return offset
end

-- shamelessly stolen from saved instances
function DungeonOrganizer:GetNextWeeklyResetTime()
	if not self.resetDays then
		local region = self:GetRegion()
		if not region then return nil end
		self.resetDays = {}
		self.resetDays.DLHoffset = 0
		if region == "US" then
			self.resetDays["2"] = true -- tuesday
			-- ensure oceanic servers over the dateline still reset on tues UTC (wed 1/2 AM server)
			self.resetDays.DLHoffset = -3 
		elseif region == "EU" then
			self.resetDays["3"] = true -- wednesday
		elseif region == "CN" or region == "KR" or region == "TW" then -- XXX: codes unconfirmed
			self.resetDays["4"] = true -- thursday
		else
			self.resetDays["2"] = true -- tuesday?
		end
	end
	local offset = (self:GetServerOffset() + self.resetDays.DLHoffset) * 3600
	local nightlyReset = self:GetNextDailyResetTime()
	if not nightlyReset then return nil end
	while not self.resetDays[date("%w",nightlyReset+offset)] do
		nightlyReset = nightlyReset + 24 * 3600
	end
	return nightlyReset
end

function DungeonOrganizer:GetNextDailyResetTime()
	local resettime = GetQuestResetTime()
	if not resettime or resettime <= 0 or -- ticket 43: can fail during startup
		-- also right after a daylight savings rollover, when it returns negative values >.<
		resettime > 24*3600+30 then -- can also be wrong near reset in an instance
		return nil
	end
	if false then -- this should no longer be a problem after the 7.0 reset time changes
		-- ticket 177/191: GetQuestResetTime() is wrong for Oceanic+Brazilian characters in PST instances
		local serverHour, serverMinute = GetGameTime()
		local serverResetTime = (serverHour*3600 + serverMinute*60 + resettime) % 86400 -- GetGameTime of the reported reset
		local diff = serverResetTime - 10800 -- how far from 3AM server
		if math.abs(diff) > 3.5*3600  -- more than 3.5 hours - ignore TZ differences of US continental servers
			and self:GetRegion() == "US" then
			local diffhours = math.floor((diff + 1800)/3600)
			resettime = resettime - diffhours*3600
			if resettime < -900 then -- reset already passed, next reset
				resettime = resettime + 86400
				elseif resettime > 86400+900 then
				resettime = resettime - 86400
			end
		end
	end
	return time() + resettime
end

local frame, events = CreateFrame("Frame"), {};
DungeonOrganizer:Init(frame)

function events:ADDON_LOADED(...)
	DungeonOrganizer:OnLoad()
	DungeonOrganizer:ResetAlts()
end
function events:CHAT_MSG_PARTY(event, message, ...)
	DungeonOrganizer:ReportData('PARTY', message)
end
function events:CHAT_MSG_PARTY_LEADER(event, message, ...)
	DungeonOrganizer:ReportData('PARTY', message)
end
function events:CHAT_MSG_GUILD(event, message, ...)
	DungeonOrganizer:ReportData('GUILD', message)
end
function events:PLAYER_LEAVING_WORLD(...)
	-- Why can't i read a key here?
	--DungeonOrganizer:CollectData()
end
function events:BAG_UPDATE_DELAYED(...)
	DungeonOrganizer:CollectData()
end
function events:PLAYER_LOGIN(...)
	DungeonOrganizer:CollectData()
end

frame:SetScript("OnEvent", function(self, event, ...)
	local arg1 = ...
	events[event](self, event, arg1);
end);
for k, v in pairs(events) do
	frame:RegisterEvent(k);
end
